<?php
/***********************************************************************

  Copyright (C) 2002, 2003, 2004  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


// The contents of this file are very much inspired by the file search.php
// from the phpBB Group forum software phpBB2 (http://www.phpbb.com). 


$pun_root = './';
require $pun_root.'include/common.php';


// Load the search.php language file
require $pun_root.'lang/'.$language.'/'.$language.'_search.php';


if (!$cookie['is_guest'])
{
	$disp_topics = $cur_user['disp_topics'];
	$disp_posts = $cur_user['disp_posts'];
}
else
{
	if ($pun_config['p_guests_read'] == '0')
		message($lang_common['Login required']);
	else if ($pun_config['p_guests_search'] == '0')
		message($lang_search['No guest search']);

	$disp_topics = $pun_config['o_disp_topics_default'];
	$disp_posts = $pun_config['o_disp_posts_default'];
}


// Detect two byte character sets
$multibyte = (in_array($lang_common['lang_encoding'], array('utf-8', 'gb2312', 'big5', 'shift_jis', 'euc-kr'))) ? true : false;


// Figure out what to do :-)
if (isset($_GET['action']) || isset($_GET['search_id']))
{
	$action = (isset($_GET['action'])) ? $_GET['action'] : null;
	$forum = (isset($_GET['forum'])) ? intval($_GET['forum']) : -1;
	$sort_dir = (isset($_GET['sort_dir'])) ? (($_GET['sort_dir'] == 'DESC') ? 'DESC' : 'ASC') : 'DESC';

	// If a search_id was supplied
	if (isset($_GET['search_id']))
	{
		$search_id = intval($_GET['search_id']);
		if (empty($search_id) || $search_id < 0)
			message($lang_common['Bad request']);
	}
	// If it's a regular search (keywords and/or author)
	else if ($action == 'search')
	{
		$keywords = (isset($_GET['keywords'])) ? trim($_GET['keywords']) : null;
		$author = (isset($_GET['author'])) ? trim($_GET['author']) : null;

		if (!$keywords && !$author)
			message($lang_search['No terms']);

		if ($author)
			$author = str_replace('*', '%', $author);

		$show_as = (isset($_GET['show_as'])) ? $_GET['show_as'] : 'posts';
		$sort_by = (isset($_GET['sort_by'])) ? intval($_GET['sort_by']) : null;
		$search_in = (!isset($_GET['search_in']) || $_GET['search_in'] == 'all') ? 0 : (($_GET['search_in'] == 'message') ? 1 : -1);
	}
	// If it's a user search (by id)
	else if ($action == 'show_user')
	{
		$user_id = intval($_GET['user_id']);
		if ($user_id < 2)
			message($lang_common['Bad request']);
	}
	else
	{
		if ($action != 'show_new' && $action != 'show_unanswered')
			message($lang_common['Bad request']);
	}


	// Fetch the list of forums
	$result = $db->query('SELECT id, forum_name, admmod_only FROM '.$db->prefix.'forums') or error('Unable to fetch forum list', __FILE__, __LINE__, $db->error());
	$num_forums = $db->num_rows($result);

	// Build two arrays with foruminfo
	$admmod_forums = array();
	for ($i = 0; $i < $num_forums; ++$i)
	{
		$forum_list[$i] = $db->fetch_row($result);
		if ($forum_list[$i][2] == '1')
			$admmod_forums[$i] = $forum_list[$i][0];	// $admmod_forums contains the ID's of admin/mod only forums
	}


	// If a valid search_id was supplied we attempt to fetch the search results from the db
	if ($search_id)
	{
		if ($cookie['is_guest'])
			$ident = get_remote_address();
		else
			$ident = addslashes($cur_user['username']);

		$result = $db->query('SELECT search_data FROM '.$db->prefix.'search_results WHERE id='.$search_id.' AND ident=\''.$ident.'\'') or error('Unable to fetch search results', __FILE__, __LINE__, $db->error());
		if ($row = $db->fetch_assoc($result))
		{
			$temp = unserialize($row['search_data']);

			$search_results = $temp['search_results'];
			$num_hits = $temp['num_hits'];
			$sort_by = $temp['sort_by'];
			$sort_dir = $temp['sort_dir'];
			$show_as = $temp['show_as'];

			unset($temp);
		}
		else
			message($lang_search['No hits']);
	}
	else
	{
		$keyword_results = $author_results = array();

		// Search a specific forum?
		if ($forum != -1)
		{
			if (in_array($forum, $admmod_forums) && $cur_user['status'] < PUN_MOD)
				message($lang_search['No hits']);

			$forum_sql = 't.forum_id = '.$forum;
		}
		else
		{
			if (empty($admmod_forums) || $cur_user['status'] > PUN_USER)
				$forum_sql = '';
			else
				$forum_sql = 't.forum_id NOT IN('.implode(',', $admmod_forums).')';
		}


		if ($author || $keywords)
		{
			// If it's a search for keywords
			if ($keywords)
			{
				$stopwords = @file('lang/'.$language.'/'.$language.'_stopwords.txt');
				$keywords = ' '.strtolower($keywords).' ';

				// Locate some common search operators
				$operator_match = array('+', '-', '&&', '||');
				$operator_replace = array(' and ', ' not ', ' and ', ' or ');
				$keywords = str_replace($operator_match, $operator_replace, $keywords);

				// Are we searching for multibyte charset text?
				if ($multibyte)
					$keywords_array = explode(' ', trim($keywords));
				else
				{
					// Filter out non-alphabetical chars
					$noise_match = array('^', '$', '&', '(', ')', '<', '>', '`', '\'', '"', '|', ',', '@', '_', '?', '%', '~', '.', '[', ']', '{', '}', ':', '\\', '/', '=', '#', '\'', ';', '!', '');
					$noise_replace = array(' ', ' ', ' ', ' ', ' ', ' ', ' ', '',  '',   ' ', ' ', ' ', ' ', '',  ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '' ,  ' ', ' ', ' ', ' ',  ' ', ' ', ' ');
					$keywords = str_replace($noise_match, $noise_replace, $keywords);

					// Filter out stopwords
					if (!empty($stopwords))
					{
						while (list(, $word) = @each($stopwords))
						{
							$word = trim($word);
							if ($word != 'and' || $word != 'or' || $word != 'not')
								$keywords = preg_replace('#\b'.preg_quote($word).'\b#', ' ', $keywords);
						}
					}

					// Split up keywords
					$keywords_array = preg_split('#[\s]+#', trim($keywords));

					// Should we search in message body or topic subject specifically?
					$search_in_cond = ($search_in) ? (($search_in > 0) ? ' AND m.subject_match = 0' : ' AND m.subject_match = 1') : '';
				}

				$match_type = 'or';
				while (list(, $cur_word) = @each($keywords_array))
				{
					switch ($cur_word)
					{
						case 'and':
						case 'or':
						case 'not':
							$match_type = $cur_word;
							break;

						default:
						{
							// Are we searching for multibyte charset text?
							if ($multibyte)
							{
								$cur_word = addslashes('%'.str_replace('*', '', $cur_word).'%');
								$cur_word_like = ($db_type == 'pgsql') ? 'ILIKE \''.$cur_word.'\'' : 'LIKE \''.$cur_word.'\'';

								if ($search_in > 0)
									$sql = 'SELECT id FROM '.$db->prefix.'posts WHERE message '.$cur_word_like;
								else if ($search_in < 0)
									$sql = 'SELECT p.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id WHERE t.subject '.$cur_word_like.' GROUP BY p.id, t.id';
								else
									$sql = 'SELECT p.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id WHERE p.message '.$cur_word_like.' OR t.subject '.$cur_word_like.' GROUP BY p.id, t.id';
							}
							else
							{
								$cur_word = str_replace('*', '%', $cur_word);
								$sql = 'SELECT m.post_id FROM '.$db->prefix.'search_words AS w INNER JOIN '.$db->prefix.'search_matches AS m ON m.word_id = w.id WHERE w.word LIKE \''.$cur_word.'\''.$search_in_cond;
							}

							$result = $db->query($sql) or error('Unable to search for posts', __FILE__, __LINE__, $db->error());

							$row = array();
							while ($temp = $db->fetch_row($result))
							{
								$row[$temp[0]] = 1;

								if (!isset($word_count))
									$result_list[$temp[0]] = 1;
								else if ($match_type == 'or')
									$result_list[$temp[0]] = 1;
								else if ($match_type == 'not')
									$result_list[$temp[0]] = 0;
							}

							if ($match_type == 'and' && $word_count)
							{
								@reset($result_list);
								while (list($post_id,) = @each($result_list))
								{
									if (!$row[$post_id])
										$result_list[$post_id] = 0;
								}
							}

							++$word_count;
							$db->free_result($result);

							break;
						}
					}
				}

				@reset($result_list);
				while (list($post_id, $matches) = @each($result_list))
				{
					if ($matches)
						$keyword_results[] = $post_id;
				}

				unset($result_list);
			}

			// If it's a search for author name (and that author name isn't Guest)
			if ($author && strcasecmp($author, 'Guest') && strcasecmp($author, $lang_common['Guest']))
			{
				switch ($db_type)
				{
					case 'mysql':
						$result = $db->query('SELECT id FROM '.$db->prefix.'users WHERE username LIKE \''.escape($author).'\'') or error('Unable to fetch users', __FILE__, __LINE__, $db->error());
						break;

					case 'pgsql':
						$result = $db->query('SELECT id FROM '.$db->prefix.'users WHERE username ILIKE \''.escape($author).'\'') or error('Unable to fetch users', __FILE__, __LINE__, $db->error());
						break;
				}

				if ($db->num_rows($result))
				{
					while ($row = $db->fetch_row($result))
						$user_ids .= (($user_ids != '') ? ',' : '').$row[0];

					$result = $db->query('SELECT id FROM '.$db->prefix.'posts WHERE poster_id IN('.$user_ids.')') or error('Unable to fetch matched posts list', __FILE__, __LINE__, $db->error());

					$search_ids = array();
					while ($row = $db->fetch_row($result))
						$author_results[] = $row[0];

					$db->free_result($result);
				}
			}


			if ($author && $keywords)
			{
				// If we searched for both keywords and author name we want the intersection between the results
				$search_ids = array_intersect($keyword_results, $author_results);
				unset($keyword_results, $author_results);
			}
			else if ($keywords)
				$search_ids = $keyword_results;
			else
				$search_ids = $author_results;

			$num_hits = count($search_ids);
			if (!$num_hits)
				message($lang_search['No hits']);


			if ($show_as == 'topics')
			{
				if ($forum_sql == '')
					$sql = 'SELECT topic_id FROM '.$db->prefix.'posts WHERE id IN('.implode(',', $search_ids).') GROUP BY topic_id';
				else
					$sql = 'SELECT p.topic_id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id WHERE p.id IN('.implode(',', $search_ids).') AND '.$forum_sql.' GROUP BY p.topic_id';

				$result = $db->query($sql) or error('Unable to fetch topic list', __FILE__, __LINE__, $db->error());

				$search_ids = array();
				while ($row = $db->fetch_row($result))
					$search_ids[] = $row[0];

				$db->free_result($result);

				$num_hits = count($search_ids);
			}
			else if ($forum_sql)
			{
				$sql = 'SELECT p.id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id WHERE p.id IN('.implode(',', $search_ids).') AND '.$forum_sql;

				$result = $db->query($sql) or error('Unable to fetch post list', __FILE__, __LINE__, $db->error());

				$search_ids = array();
				while ($row = $db->fetch_row($result))
					$search_ids[] = $row[0];

				$db->free_result($result);

				$num_hits = count($search_ids);
			}
		}
		else if ($action == 'show_new' || $action == 'show_user' || $action == 'show_unanswered')
		{
			// If it's a search for new posts
			if ($action == 'show_new')
			{
				if ($cookie['is_guest'])
					message($lang_common['No permission']);

				if ($forum_sql != '')
					$sql = 'SELECT t.id FROM '.$db->prefix.'topics AS t WHERE t.last_post>'.$cur_user['last_visit'].' AND '.$forum_sql;
				else
					$sql = 'SELECT id FROM '.$db->prefix.'topics WHERE last_post>'.$cur_user['last_visit'];

				$result = $db->query($sql) or error('Unable to fetch topic list', __FILE__, __LINE__, $db->error());
				$num_hits = $db->num_rows($result);

				if (!$num_hits)
					message($lang_search['No new posts']);
			}
			// If it's a search for posts by a specific user ID
			else if ($action == 'show_user')
			{
				if ($forum_sql != '')
					$sql = 'SELECT t.id FROM '.$db->prefix.'topics AS t INNER JOIN '.$db->prefix.'posts AS p ON p.topic_id=t.id WHERE p.poster_id='.$user_id.' AND '.$forum_sql.' GROUP BY t.id';
				else
					$sql = 'SELECT t.id FROM '.$db->prefix.'topics AS t INNER JOIN '.$db->prefix.'posts AS p ON p.topic_id=t.id WHERE p.poster_id='.$user_id.' GROUP BY t.id';

				$result = $db->query($sql) or error('Unable to fetch topic list', __FILE__, __LINE__, $db->error());
				$num_hits = $db->num_rows($result);

				if (!$num_hits)
					message($lang_search['User no posts']);
			}
			// If it's a search for unanswered posts
			else
			{
				if ($forum_sql != '')
					$sql = 'SELECT t.id FROM '.$db->prefix.'topics AS t WHERE t.num_replies=0 AND t.moved_to IS NULL AND '.$forum_sql;
				else
					$sql = 'SELECT id FROM '.$db->prefix.'topics WHERE num_replies=0 AND moved_to IS NULL';

				$result = $db->query($sql) or error('Unable to fetch topic list', __FILE__, __LINE__, $db->error());
				$num_hits = $db->num_rows($result);

				if (!$num_hits)
					message($lang_search['No unanswered']);
			}

			// We want to sort things after last post
			$sort_by = 4;

			$search_ids = array();
			while ($row = $db->fetch_row($result))
				$search_ids[] = $row[0];

			$db->free_result($result);

			$show_as = 'topics';
		}
		else
			message($lang_common['Bad request']);


		// Prune "old" search results
		$result = $db->query('SELECT ident FROM '.$db->prefix.'online') or error('Unable to fetch online list', __FILE__, __LINE__, $db->error());

		if ($db->num_rows($result) > 0)
		{
			while ($row = $db->fetch_row($result))
				$old_searches[] = '\''.addslashes($row[0]).'\'';

			$db->query('DELETE FROM '.$db->prefix.'search_results WHERE ident NOT IN('.implode(',', $old_searches).')') or error('Unable to delete search results', __FILE__, __LINE__, $db->error());
		}
		// Final search results
		$search_results = implode(',', $search_ids);

		// Fill an array with our results and search properties
		$temp['search_results'] = $search_results;
		$temp['num_hits'] = $num_hits;
		$temp['sort_by'] = $sort_by;
		$temp['sort_dir'] = $sort_dir;
		$temp['show_as'] = $show_as;
		$temp = addslashes(serialize($temp));
		$search_id = mt_rand();

		if ($cookie['is_guest'])
			$ident = get_remote_address();
		else
			$ident = addslashes($cur_user['username']);

		$db->query('UPDATE '.$db->prefix.'search_results SET id='.$search_id.', search_data=\''.$temp.'\' WHERE ident=\''.$ident.'\'') or error('Unable to update search results', __FILE__, __LINE__, $db->error());
		if (!$db->affected_rows())
			$db->query('INSERT INTO '.$db->prefix.'search_results (id, ident, search_data) VALUES('.$search_id.', \''.$ident.'\', \''.$temp.'\')') or error('Unable to insert search results', __FILE__, __LINE__, $db->error());
	}


	// Fetch results to display
	if ($search_results != '')
	{
		switch ($sort_by)
		{
			case 1:
				$sql = ($show_as == 'topics') ? 't.poster' : 'p.poster';
				break;

			case 2:
				$sql = 't.subject';
				break;

			case 3:
				$sql = 't.forum_id';
				break;

			case 4:
				$sql = 't.last_post';
				break;

			default:
			{
				$sql = ($show_as == 'topics') ? 't.posted' : 'p.posted';

				if ($show_as == 'topics')
					$group_by = ', t.posted';

				break;
			}
		}

		if ($show_as == 'posts')
			$sql = 'SELECT p.id AS pid, p.poster AS pposter, p.poster_id, SUBSTRING(p.message, 1, 140) AS message, t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.forum_id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id WHERE p.id IN('.$search_results.') ORDER BY '.$sql;
		else
			$sql = 'SELECT t.id AS tid, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.forum_id FROM '.$db->prefix.'posts AS p INNER JOIN '.$db->prefix.'topics AS t ON p.topic_id=t.id WHERE t.id IN('.$search_results.') GROUP BY t.id, t.poster, t.subject, t.last_post, t.last_post_id, t.last_poster, t.num_replies, t.forum_id'.$group_by.' ORDER BY '.$sql;


		// Determine the topic or post offset (based on $_GET['p'])
		$per_page = ($show_as == 'posts') ? $disp_posts : $disp_topics;
		$num_pages = ceil($num_hits / $per_page);

		$p = (!isset($_GET['p']) || $_GET['p'] <= 1 || $_GET['p'] > $num_pages) ? 1 : $_GET['p'];
		$start_from = $per_page * ($p - 1);


		$sql .= ' '.$sort_dir.' LIMIT '.$start_from.', '.$per_page;

		$result = $db->query($sql) or error('Unable to fetch search results', __FILE__, __LINE__, $db->error());

		$search_set = array();
		while ($row = $db->fetch_assoc($result))
			$search_set[] = $row;

		$db->free_result($result);


		$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / '.$lang_search['Search results'];
		require $pun_root.'header.php';

?>
<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>

<table class="punmain" cellspacing="1" cellpadding="4">
	<tr class="punhead">
		<td class="punhead" style="width: 24px">&nbsp;</td>
		<td class="punhead" style="white-space: nowrap"><?php echo ($show_as == 'posts') ? $lang_search['Topic/Message'] : $lang_common['Topic']; ?></td>
		<td class="punhead" style="width: 18%; white-space: nowrap"><?php echo $lang_common['Forum'] ?></td>
		<td class="punhead" style="width: 11%; white-space: nowrap"><?php echo $lang_common['Author'] ?></td>
		<td class="punheadcent" style="width: 7%; white-space: nowrap"><?php echo $lang_common['Replies'] ?></td>
		<td class="punhead" style="width: 25%; white-space: nowrap"><?php echo $lang_common['Last post'] ?></td>
	</tr>
<?php

		for ($i = 0; $i < count($search_set); ++$i)
		{
			@reset($forum_list);
			while (list(, $temp) = @each($forum_list))
			{
				if ($temp[0] == $search_set[$i]['forum_id'])
					$forum = '<a href="viewforum.php?id='.$temp[0].'">'.$temp[1].'</a>';
			}

			if ($pun_config['o_censoring'] == '1')
				$search_set[$i]['subject'] = censor_words($search_set[$i]['subject']);

			$subject = '<a href="viewtopic.php?id='.$search_set[$i]['tid'].'">'.pun_htmlspecialchars($search_set[$i]['subject']).'</a>';

			if (!$cookie['is_guest'] && $search_set[$i]['last_post'] > $cur_user['last_visit'])
			{
				if ($cur_user['show_img'] == '1')
					$icon = '<img src="img/'.$cur_user['style'].'_new.png" width="16" height="16" alt="">';
				else
					$icon = '<span class="puntext"><b>&#8226;</b></span>';

				$subject = '<b>'.$subject.'</b>';
				$subject_new_posts = '[&nbsp;<a href="viewtopic.php?id='.$search_set[$i]['tid'].'&amp;action=new" title="'.$lang_common['New posts info'].'">'.$lang_common['New posts'].'</a>&nbsp;]';
			}
			else
			{
				$icon = '&nbsp;';
				$subject_new_posts = null;
			}

			if ($show_as == 'posts')
			{
				if ($pun_config['o_censoring'] == '1')
					$search_set[$i]['message'] = censor_words($search_set[$i]['message']);

				$message = str_replace("\n", '<br>', pun_htmlspecialchars($search_set[$i]['message']));
				$pposter = pun_htmlspecialchars($search_set[$i]['pposter']);

				if ($search_set[$i]['poster_id'] > 1)
					$pposter = '<a href="profile.php?id='.$search_set[$i]['poster_id'].'">'.$pposter.'</a>';

				if (pun_strlen($message) == 140)
					$message .= ' ...';

?>
	<tr class="puntopic">
		<td class="puncon1cent"><?php echo $icon ?></td>
		<td class="puncon2">
			<?php echo $lang_common['Topic'] ?>: <?php echo $subject ?><br>
			<?php echo $lang_common['Author'] ?>: <?php echo $pposter ?><br><br>
			<table class="punplain" style="table-layout: fixed" cellspacing="4" cellpadding="6">
				<tr>
					<td class="punquote">
						<?php echo $message ?>
						<div style="text-align: right"><a href="viewtopic.php?pid=<?php echo $search_set[$i]['pid'].'#'.$search_set[$i]['pid'] ?>"><?php echo $lang_search['Go to post'] ?></a></div>
					</td>
				</tr>
			</table>
		</td>
		<td class="puncon1"><?php echo $forum ?></td>
		<td class="puncon2"><?php echo pun_htmlspecialchars($search_set[$i]['poster']) ?></td>
		<td class="puncon1cent"><?php echo $search_set[$i]['num_replies'] ?></td>
		<td class="puncon2" style="white-space: nowrap"><?php echo '<a href="viewtopic.php?pid='.$search_set[$i]['last_post_id'].'#'.$search_set[$i]['last_post_id'].'">'.format_time($search_set[$i]['last_post']).'</a> '.$lang_common['by'].' '.pun_htmlspecialchars($search_set[$i]['last_poster']) ?></td>
	</tr>
<?php

			}
			else
			{
				$num_pages_topic = ceil(($search_set[$i]['num_replies'] + 1) / $disp_posts);

				if ($num_pages_topic > 1)
				{
					$stop = ($num_pages_topic < 3) ? ($num_pages_topic + 1) : 4;

					$subject_multipage = '[';
					for ($current = 1; $current < $stop; ++$current)
						$subject_multipage .= '&nbsp;<a href="viewtopic.php?id='.$search_set[$i]['tid'].'&amp;p='.$current.'">'.$current.'</a>';

					if ($num_pages_topic > 3)
						$subject_multipage .= '&nbsp;...&nbsp;<a href="viewtopic.php?id='.$search_set[$i]['tid'].'&amp;p='.$num_pages_topic.'">'.$num_pages_topic.'</a>&nbsp;]';
					else
						$subject_multipage .= '&nbsp;]';
				}
				else
					$subject_multipage = null;

				// Should we show the "New posts" and/or the multipage links?
				if (!empty($subject_new_posts) || !empty($subject_multipage))
				{
					$subject .= '&nbsp; '.(!empty($subject_new_posts) ? $subject_new_posts : '');
					$subject .= !empty($subject_multipage) ? ' '.$subject_multipage : '';
				}

?>
	<tr class="puntopic">
		<td class="puncon1cent"><?php echo $icon ?></td>
		<td class="puncon2"><?php echo $subject ?></td>
		<td class="puncon1"><?php echo $forum ?></td>
		<td class="puncon2"><?php echo pun_htmlspecialchars($search_set[$i]['poster']) ?></td>
		<td class="puncon1cent"><?php echo $search_set[$i]['num_replies'] ?></td>
		<td class="puncon2" style="white-space: nowrap"><?php echo '<a href="viewtopic.php?pid='.$search_set[$i]['last_post_id'].'#'.$search_set[$i]['last_post_id'].'">'.format_time($search_set[$i]['last_post']).'</a> '.$lang_common['by'].' '.pun_htmlspecialchars($search_set[$i]['last_poster']) ?></td>
	</tr>
<?php

			}
		}

?>
</table>

<table class="punspacer" cellspacing="1" cellpadding="4">
	<tr>
		<td><?php echo $lang_common['Pages'].': '.paginate($num_pages, $p, 'search.php?search_id='.$search_id) ?></td>
	</tr>
</table>
<?php

		$footer_style = 'search';
		require $pun_root.'footer.php';
	}
	else
		message($lang_search['No hits']);
}


if ($pun_config['o_search'] == '0' && $cur_user['status'] < PUN_MOD)
	message($lang_search['Search disabled']);

$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / '.$lang_search['Search'];
$validate_form = true;
$form_name = 'search';
$focus_element = 'keywords';
require $pun_root.'header.php';

?>
<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>

<form method="get" action="search.php" id="search" onsubmit="return process_form(this)">
	<input type="hidden" name="action" value="search">
	<table class="punmain" cellspacing="1" cellpadding="4">
		<tr class="punhead">
			<td class="punhead" colspan="2"><?php echo $lang_search['Search'] ?></td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><b><?php echo $lang_search['Keyword search'] ?></b>&nbsp;&nbsp;</td>
			<td class="puncon2">
				<?php echo $lang_search['Keyword search info'] ?><br><br>
				&nbsp;<input type="text" name="keywords" size="40" maxlength="100">
			</td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><b><?php echo $lang_search['Author search'] ?></b>&nbsp;&nbsp;</td>
			<td class="puncon2">
				<?php echo $lang_search['Author search info'] ?><br><br>
				&nbsp;<input type="text" name="author" size="40" maxlength="100">
			</td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><b><?php echo $lang_search['Forum search'] ?></b>&nbsp;&nbsp;</td>
			<td class="puncon2">
				<br>&nbsp;
				<select name="forum">
<?php

if ($pun_config['o_search_all_forums'] == '1' || $cur_user['status'] > PUN_USER)
	echo "\t\t\t\t\t".'<option value="-1">'.$lang_search['All forums'].'</option>'."\n";


$extra_sql = ($cur_user['status'] < PUN_MOD) ? ' WHERE f.admmod_only=\'0\'' : '';

$result = $db->query('SELECT c.id AS cid, c.cat_name, f.id AS fid, f.forum_name FROM '.$db->prefix.'categories AS c INNER JOIN '.$db->prefix.'forums AS f ON c.id=f.cat_id'.$extra.' ORDER BY c.disp_position, c.id, f.disp_position') or error('Unable to fetch category/forum list', __FILE__, __LINE__, $db->error());
$num_forums = $db->num_rows($result);

while ($num_forums--)
{
	$forum = $db->fetch_assoc($result);

	if ($forum['cid'] != $cur_category)	// Are we still in the same category?
	{
		if (!empty($cur_category))
			echo "\t\t\t\t\t".'</optgroup>'."\n";

		echo "\t\t\t\t\t".'<optgroup label="'.pun_htmlspecialchars($forum['cat_name']).'">'."\n";
		$cur_category = $forum['cid'];
	}

	echo "\t\t\t\t\t\t".'<option value="'.$forum['fid'].'">'.pun_htmlspecialchars($forum['forum_name']).'</option>'."\n";
}

?>
					</optgroup>
				</select><br><br>
			</td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><b><?php echo $lang_search['Search in'] ?></b>&nbsp;&nbsp;</td>
			<td class="puncon2">
				&nbsp;&nbsp;<input type="radio" name="search_in" value="all" checked>&nbsp;<?php echo $lang_search['Message and subject'] ?><br>
				&nbsp;&nbsp;<input type="radio" name="search_in" value="message">&nbsp;<?php echo $lang_search['Message only'] ?><br>
				&nbsp;&nbsp;<input type="radio" name="search_in" value="topic">&nbsp;<?php echo $lang_search['Topic only'] ?>
			</td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><b><?php echo $lang_search['Sort by'] ?></b>&nbsp;&nbsp;</td>
			<td class="puncon2">
				<br>&nbsp;
				<select name="sort_by">
					<option value="0"><?php echo $lang_search['Sort by post time'] ?></option>
					<option value="1"><?php echo $lang_search['Sort by author'] ?></option>
					<option value="2"><?php echo $lang_search['Sort by subject'] ?></option>
					<option value="3"><?php echo $lang_search['Sort by forum'] ?></option>
				</select>
				&nbsp;&nbsp;<input type="radio" name="sort_dir" value="ASC">&nbsp;<?php echo $lang_search['Ascending'] ?>
				&nbsp;&nbsp;<input type="radio" name="sort_dir" value="DESC" checked>&nbsp;<?php echo $lang_search['Descending'] ?><br><br>
			</td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><b><?php echo $lang_search['Show as'] ?></b>&nbsp;&nbsp;</td>
			<td class="puncon2">
				&nbsp;&nbsp;<input type="radio" name="show_as" value="topics" checked>&nbsp;<?php echo $lang_search['Show as topics'] ?><br>
				&nbsp;&nbsp;<input type="radio" name="show_as" value="posts">&nbsp;<?php echo $lang_search['Show as posts'] ?>
			</td>
		</tr>
		<tr>
			<td class="puncon1right" style="width: 140px; white-space: nowrap"><?php echo $lang_common['Actions'] ?>&nbsp;&nbsp;</td>
			<td class="puncon2"><br>&nbsp;&nbsp;<input type="submit" name="search" value="<?php echo $lang_common['Submit'] ?>" accesskey="s"><br><br></td>
		</tr>
	</table>
</form>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>
<?php

require $pun_root.'footer.php';
